import json
import os
import sys
import re

SCRIPT_VERSION = 1

# script to pull interesting data out of sudoers files

def ParseSudoersFile(filepath, defaults, users, groups):
    with open(filepath, 'rt') as f:
        for entry in f.readlines():
            entry = entry.lstrip().rstrip('\n')

            if entry.startswith('#'):
                continue

            # collect defaults
            if entry.lower().startswith('defaults'):
                tokens = re.split('\s+',entry, 1)
                if tokens:
                    defaults.append(tokens[1])
                continue

            tokens = entry.split()
            if not tokens:
                continue

            # collect groups
            if entry.startswith('%'):
                groups.append(tokens[0][1:])
                continue

            # collect users or commands
            for token in tokens[0].split(","):
                # ignore commands
                if not os.path.isfile(token):
                    users.append(token)

    return defaults, users, groups

sudoers_file = '/etc/sudoers'
sudoers_dir = '/etc/sudoers.d'

defaults, users, groups = ParseSudoersFile(sudoers_file,[],[],[])
for filename in os.listdir(sudoers_dir):
    path = os.path.join(sudoers_dir, filename)
    if not os.path.isfile(path):
        continue
    defaults, users, groups = ParseSudoersFile(path, defaults, users, groups)

def ConstructOutput(defaults, users, groups):
    return {
        'scriptVersion' : SCRIPT_VERSION,
        'defaults' : list(set(defaults)),
        'users' : list(set(users)),
        'groups' : list(set(groups))
    }
    
output = ConstructOutput(defaults, users, groups)
sys.stdout.write(json.dumps(output))
